==> unicore-uvos-server-1.5.1-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/registration/RegistrationHandler.java <==
/*
 * Copyright (c) 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Jan 9, 2009
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.registration;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.jws.WebService;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlCursor;
import org.oasisOpen.docs.wsrf.sg2.AddResponseDocument;
import org.w3.x2005.x08.addressing.AttributedURIType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3.x2005.x08.addressing.MetadataType;

import pl.edu.icm.unicore.uvos.util.ConfDefaults;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLQueryInterface;

public class RegistrationHandler
{
	public final static QName INTERFACE_NAME=
		new QName("http://www.w3.org/2005/08/addressing/metadata",
				"InterfaceName");
	private static final Logger logger = Logger.getLogger(RegistrationHandler.class);
	private List<RegistryClient> clients = new ArrayList<RegistryClient>();
	private EndpointReferenceType serviceEPR;

	/**
	 * add interface name of a service to the metadata of the epr
	 */
	public static void addPortType(EndpointReferenceType epr, QName portType)
	{
		MetadataType meta=epr.getMetadata();
		if(meta==null) meta=epr.addNewMetadata();
		XmlCursor n=meta.newCursor();
		n.toFirstContentToken();
		n.beginElement(INTERFACE_NAME);
		n.insertNamespace("x", portType.getNamespaceURI());
		n.insertChars("x:"+portType.getLocalPart());
		n.dispose();
	}

	
	public RegistrationHandler(List<String> registries, Configuration conf)
	{
		serviceEPR = EndpointReferenceType.Factory.newInstance();
		
		AttributedURIType addr = serviceEPR.addNewAddress();
		
		WebService samlQueryWS = 
			SAMLQueryInterface.class.getAnnotation(WebService.class);
		String samlQueryServiceName = samlQueryWS.serviceName();

		boolean httpsEnable = conf.getAsBool(ConfDefaults.HTTPS_ENABLE, 
			ConfDefaults.httpsEnable, null);
		if (!httpsEnable)
		{
			logger.warn("Not adding service to registry as HTTPS connector is disabled.");
			return;
		}
		String host = conf.getAsString(ConfDefaults.HTTPS_HOST, 
			ConfDefaults.httpsHost, null);
		int port = conf.getAsInt(ConfDefaults.HTTPS_PORT, 
			ConfDefaults.httpsPort, null);
		addr.setStringValue("https://" + host + ":" + port + "/" + samlQueryServiceName);
		
		addPortType(serviceEPR, new QName(samlQueryWS.targetNamespace(), 
			samlQueryServiceName));
		
		for (String reg: registries)
		{
			try
			{
				clients.add(new RegistryClient(reg, conf));
			} catch (Exception e)
			{
				logger.error("Can't create client for registry " + reg, e);
			}
		}
		Adder thread = new Adder();
		thread.setDaemon(true);
		thread.start();
	}
	
	public class Adder extends Thread
	{
		public void run()
		{
			do
			{
				Calendar next = readd();
				Calendar now = Calendar.getInstance();
				long wait = next.getTimeInMillis() - now.getTimeInMillis();
				logger.debug("Delay for registry update is: " + wait);
				try
				{
					Thread.sleep(wait);
				} catch (InterruptedException e) {}
			} while(true);
		}
		
		private Calendar readd()
		{
			Calendar minTT = null;
			for (RegistryClient client: clients)
			{
				try
				{
					AddResponseDocument resp = client.addRegistryEntry(serviceEPR);
					logger.info("Successfully registered service at registry " + 
						client.getEPR().getAddress().getStringValue());
					Calendar tt = resp.getAddResponse().getTerminationTime();
					if (tt!= null && (minTT == null || tt.before(minTT)))
						minTT = tt;
				} catch (Exception e)
				{
					logger.error("Can't add service to registry " + 
						client.getEPR().getAddress().getStringValue(), e);
				}
			}
			if (minTT == null)
			{
				minTT = Calendar.getInstance();
				minTT.add(Calendar.MINUTE, 5);
			}
			return minTT;
		}
	}
}

==> unicore-uvos-server-1.6.0-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/registration/RegistrationHandler.java <==
/*
 * Copyright (c) 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Jan 9, 2009
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.registration;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.jws.WebService;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlCursor;
import org.oasisOpen.docs.wsrf.sg2.AddResponseDocument;
import org.w3.x2005.x08.addressing.AttributedURIType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3.x2005.x08.addressing.MetadataType;

import eu.unicore.util.Log;

import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;
import pl.edu.icm.unicore.uvos.util.ThreadScheduler;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLQueryInterface;

public class RegistrationHandler
{
	public final static QName INTERFACE_NAME=
		new QName("http://www.w3.org/2005/08/addressing/metadata",
				"InterfaceName");
	private static final Logger logger = Log.getLogger(UVOSServerProperties.LOG_PFX, RegistrationHandler.class);
	private List<RegistryClient> clients = new ArrayList<RegistryClient>();
	private EndpointReferenceType serviceEPR;
	private ScheduledExecutorService scheduler;

	public RegistrationHandler(List<String> registries, UVOSServerProperties conf, ThreadScheduler thScheduler)
	{
		serviceEPR = EndpointReferenceType.Factory.newInstance();
		
		AttributedURIType addr = serviceEPR.addNewAddress();
		
		WebService samlQueryWS = 
			SAMLQueryInterface.class.getAnnotation(WebService.class);
		String samlQueryServiceName = samlQueryWS.serviceName();

		boolean httpsEnable = conf.getBooleanValue(UVOSServerProperties.HTTPS_ENABLE);
		if (!httpsEnable)
		{
			logger.warn("Not adding service to registry as HTTPS connector is disabled.");
			return;
		}
		String host = conf.getValue(UVOSServerProperties.HTTPS_HOST);
		String port = conf.getValue(UVOSServerProperties.HTTPS_PORT);
		addr.setStringValue("https://" + host + ":" + port + "/" + samlQueryServiceName);
		
		addPortType(serviceEPR, new QName(samlQueryWS.targetNamespace(), 
			samlQueryServiceName));
		
		for (String reg: registries)
		{
			try
			{
				clients.add(new RegistryClient(reg, conf));
			} catch (Exception e)
			{
				logger.error("Can't create client for registry " + reg, e);
			}
		}
		scheduler = thScheduler.getScheduledExecutor();
	}

	public void start()
	{
		Adder adder = new Adder();
		scheduler.schedule(adder, 1, TimeUnit.SECONDS);
	}
	
	
	/**
	 * add interface name of a service to the metadata of the epr
	 */
	public static void addPortType(EndpointReferenceType epr, QName portType)
	{
		MetadataType meta=epr.getMetadata();
		if(meta==null) meta=epr.addNewMetadata();
		XmlCursor n=meta.newCursor();
		n.toFirstContentToken();
		n.beginElement(INTERFACE_NAME);
		n.insertNamespace("x", portType.getNamespaceURI());
		n.insertChars("x:"+portType.getLocalPart());
		n.dispose();
	}

	
	
	public class Adder implements Runnable
	{
		public void run()
		{
			Calendar next = readd();
			Calendar now = Calendar.getInstance();
			long wait = (next.getTimeInMillis() - now.getTimeInMillis())/2;
			logger.debug("Delay for registry update is: " + wait);
			scheduler.schedule(this, wait, TimeUnit.MILLISECONDS);
		}
		
		private Calendar readd()
		{
			Calendar minTT = null;
			for (RegistryClient client: clients)
			{
				try
				{
					AddResponseDocument resp = client.addRegistryEntry(serviceEPR);
					logger.info("Successfully registered service at registry " + 
						client.getEPR().getAddress().getStringValue());
					Calendar tt = resp.getAddResponse().getTerminationTime();
					if (tt!= null && (minTT == null || tt.before(minTT)))
						minTT = tt;
				} catch (Exception e)
				{
					logger.error("Can't add service to registry " + 
						client.getEPR().getAddress().getStringValue(), e);
				}
			}
			if (minTT == null)
			{
				minTT = Calendar.getInstance();
				minTT.add(Calendar.MINUTE, 5);
			}
			return minTT;
		}
	}
}
